@@ -64,7 +64,14 @@
 
 //kernel time resolution (inverse of one jiffy interval) in Hertz
 //i don't know how to detect it, then define to the default (not very clean!)
+// On Linux the _default_ is now 250, so we will use that on Linux. Will assume
+// 100 on other platforms, but this can be patched on a per build system basis.
+#ifdef LINUX
+#define HZ 250
+#else
 #define HZ 100
+#endif
+int my_HZ = HZ;
 
 //some useful macro
 #define min(a,b) (a<b?a:b)
@@ -522,7 +529,7 @@ int compute_cpu_usage(int pid,int last_working_quantum,struct cpu_usage *pusage)
 			i=(i+1)%MEM_ORDER;
 		} while (i!=max);
 		long used=ps[front].jiffies-ps[tail].jiffies;
-		float usage=(used*1000000.0/HZ)/dtwork;
+		float usage=(used*1000000.0/my_HZ)/dtwork;
 		pusage->workingrate=1.0*dtwork/dt;
 		pusage->pcpu=usage*pusage->workingrate;
 		if (size==MEM_ORDER)
@@ -811,6 +818,7 @@ void *Monitor_Children(void *all_data)
 
 void print_usage(FILE *stream,int exit_code) {
         fprintf(stream, "CPUlimit version %1.1f\n", VERSION);
+        fprintf(stream, "CPUlimit default HZ value %d\n", HZ);
 	fprintf(stream, "Usage: %s TARGET [OPTIONS...] [-- PROGRAM]\n",program_name);
 	fprintf(stream, "   TARGET must be exactly one of these:\n");
 	fprintf(stream, "      -p, --pid=N        pid of the process\n");
@@ -823,6 +831,7 @@ void print_usage(FILE *stream,int exit_code) {
         fprintf(stream, "      -b  --background   run in background\n");
         fprintf(stream, "      -f  --foreground   launch target process in foreground and wait for it to exit\n");
         fprintf(stream, "      -c  --cpu=N        override the detection of CPUs on the machine.\n");
+        fprintf(stream, "      -j  --hz=N         override kernel HZ value (defaults to %d).\n", HZ);
 	fprintf(stream, "      -l, --limit=N      percentage of cpu allowed from 1 up.\n");
         fprintf(stream, "                         Usually 1 - %d00, but can be higher\n", NCPU);
         fprintf(stream, "                         on multi-core CPUs (mandatory)\n");
@@ -899,16 +908,17 @@ int main(int argc, char **argv) {
 	int next_option;
 	/* A string listing valid short options letters. */
         #ifdef LINUX
-	const char* short_options="p:e:P:l:c:s:bfqkmrvzh";
+	const char* short_options="p:e:P:j:l:c:s:bfqkmrvzh";
         PROGRAM_DATA program_data;
         #else
-	const char* short_options="p:e:P:l:c:s:bfqkrvzh";
+	const char* short_options="p:e:P:j:l:c:s:bfqkrvzh";
         #endif
 	/* An array describing valid long options. */
 	const struct option long_options[] = {
 		{ "pid", required_argument, NULL, 'p' },
 		{ "exe", required_argument, NULL, 'e' },
 		{ "path", required_argument, NULL, 'P' },
+		{ "hz", required_argument, NULL, 'j' },
 		{ "limit", required_argument, NULL, 'l' },
                 { "background", no_argument, NULL, 'b' },
                 { "foreground", no_argument, NULL, 'f' },
@@ -941,7 +951,7 @@ int main(int argc, char **argv) {
 
         opterr = 0;      // avoid unwanted error messages for unknown parameters
 	do {
-		next_option = getopt_long (argc, argv, short_options,long_options, NULL);
+		next_option = getopt_long (argc, argv, short_options, long_options, NULL);
 		switch(next_option) {
                         case 'b':
                                 run_in_background = TRUE;
@@ -971,6 +981,10 @@ int main(int argc, char **argv) {
 				process_ok = TRUE;
                                 last_known_argument += 2;
 				break;
+                        case 'j':
+                                my_HZ = atoi(optarg);
+                                last_known_argument += 2;
+                                break;
 			case 'l':
 				perclimit=atoi(optarg);
 				limit_ok = TRUE;
@@ -1035,6 +1049,8 @@ int main(int argc, char **argv) {
 	} while(next_option != -1);
 
         signal(SIGCHLD, Child_Done);
+        if (verbose)
+           printf("Proceeding with kernel frequency set to %d\n", my_HZ);
 
         // try to launch a program passed on the command line
         // But only if we do not already have a PID to watch
